@@ -607,49 +607,36 @@ class _Scheduler(object):
     def scheduler_task(self, conn, addr, coro=None):
         # generator
         def _job_request_task(self, msg):
-            # generator
+            # function
             try:
                 _job = unserialize(msg)
             except:
                 logger.debug('Ignoring job request from %s', addr[0])
-                raise StopIteration
+                return
             _job.uid = id(_job)
             setattr(_job, 'node', None)
             job = type('DispyJob', (), {'status':DispyJob.Created,
                                         'start_time':None, 'end_time':None})
             setattr(_job, 'job', job)
-            resp = serialize(_job.uid)
-            try:
-                yield conn.send_msg(resp)
-            except:
-                logger.warning('Failed to send response to %s: %s',
-                               str(addr), traceback.format_exc())
-                raise StopIteration
-            try:
-                msg = yield conn.recv_msg()
-                assert msg == 'ACK'
-            except:
-                logger.warning('Invalid reply for job: %s, %s',
-                               _job.uid, traceback.format_exc())
-                raise StopIteration
             cluster = self._clusters.get(_job.compute_id, None)
             if cluster is None:
                 logger.debug('cluster %s is not valid anymore for job %s',
                              _job.compute_id, _job.uid)
-                raise StopIteration
+                return
             cluster._jobs.append(_job)
             self.unsched_jobs += 1
             cluster._pending_jobs += 1
             cluster.last_pulse = time.time()
             self._sched_event.set()
+            return serialize(_job.uid)
 
         def _compute_task(self, msg):
-            # generator
+            # function
             try:
                 compute = unserialize(msg)
             except:
                 logger.debug('Ignoring compute request from %s', addr[0])
-                raise StopIteration(None)
+                return
             setattr(compute, 'nodes', {})
             cluster = _Cluster(compute)
             cluster.ip_addr = conn.getsockname()[0]
@@ -671,22 +658,19 @@ class _Scheduler(object):
                 except:
                     logger.warning('Could not create directory "%s"', dest_path)
                     if compute.xfer_files:
-                        conn.close()
-                        raise StopIteration
+                        return
             for xf in compute.xfer_files:
                 if self.max_file_size and xf.stat_buf.st_size > self.max_file_size:
                     logger.warning('transfer file "%s" is too big (%s)',
                                    xf.name, xf.stat_buf.st_size)
-                    conn.close()
-                    raise StopIteration
+                    return
                 xf.compute_id = compute.id
                 xf.name = os.path.join(dest_path, os.path.basename(xf.name))
-            resp = {'ID':compute.id, 'pulse_interval':self.pulse_interval,
-                    'job_result_port':compute.job_result_port}
-            resp = serialize(resp)
             logger.debug('New computation %s: %s, %s', compute.id, compute.name,
                          len(compute.xfer_files))
-            yield resp
+            resp = {'ID':compute.id, 'pulse_interval':self.pulse_interval,
+                    'job_result_port':compute.job_result_port}
+            return serialize(resp)
 
         def _xfer_file_task(self, msg):
             # generator
@@ -694,18 +678,17 @@ class _Scheduler(object):
                 xf = unserialize(msg)
             except:
                 logger.debug('Ignoring file trasnfer request from %s', addr[0])
-                raise StopIteration
-            resp = ''
+                raise StopIteration('NAK')
             if xf.compute_id not in self._clusters:
                 logger.error('computation "%s" is invalid' % xf.compute_id)
-                raise StopIteration
+                raise StopIteration('NAK')
             compute = self._clusters[xf.compute_id]
             dest_path = os.path.join(self.dest_path_prefix, str(compute.id))
             if not os.path.isdir(dest_path):
                 try:
                     os.makedirs(dest_path)
                 except:
-                    raise StopIteration
+                    raise StopIteration('NAK')
             tgt = os.path.join(dest_path, os.path.basename(xf.name))
             logger.debug('Copying file %s to %s (%s)', xf.name, tgt, xf.stat_buf.st_size)
             try:
@@ -724,19 +707,21 @@ class _Scheduler(object):
                 if n < xf.stat_buf.st_size:
                     resp = 'NAK (read only %s bytes)' % n
                 else:
-                    resp = 'ACK'
-                    logger.debug('Copied file %s, %s', tgt, resp)
                     os.utime(tgt, (xf.stat_buf.st_atime, xf.stat_buf.st_mtime))
                     os.chmod(tgt, stat.S_IMODE(xf.stat_buf.st_mode))
+                    logger.debug('Copied file %s', tgt)
+                    resp = 'ACK'
             except:
                 logger.warning('Copying file "%s" failed with "%s"',
                                xf.name, traceback.format_exc())
-                resp = 'NACK'
-            if resp:
+                resp = 'NAK'
                 try:
-                    yield conn.send_msg(resp)
+                    os.remove(tgt)
+                    if len(os.listdir(dest_path)) == 0:
+                        os.rmdir(dest_path)
                 except:
-                    logger.debug('Could not send reply for "%s"', xf.name)
+                    pass
+            raise StopIteration(resp)
 
         # scheduler_task begins here
         conn.settimeout(2)
@@ -769,16 +754,16 @@ class _Scheduler(object):
             raise StopIteration
         msg = yield conn.recv_msg()
         if not msg:
-            conn.close()
             logger.info('Closing connection')
+            conn.close()
             raise StopIteration
+
         if msg.startswith('JOB:'):
             msg = msg[len('JOB:'):]
-            yield _job_request_task(self, msg)
-            conn.close()
+            resp = _job_request_task(self, msg)
         elif msg.startswith('COMPUTE:'):
             msg = msg[len('COMPUTE:'):]
-            resp = yield _compute_task(self, msg)
+            resp = _compute_task(self, msg)
         elif msg.startswith('ADD_COMPUTE:'):
             msg = msg[len('ADD_COMPUTE:'):]
             try:
@@ -787,40 +772,41 @@ class _Scheduler(object):
                 for xf in cluster._compute.xfer_files:
                     assert os.path.isfile(xf.name)
                 resp = serialize(cluster._compute.id)
-                Coro(self.add_cluster, cluster)
             except:
                 logger.debug('Ignoring compute request from %s', addr[0])
-                conn.close()
-                raise StopIteration
+            else:
+                Coro(self.add_cluster, cluster)
         elif msg.startswith('DEL_COMPUTE:'):
             msg = msg[len('DEL_COMPUTE:'):]
-            conn.close()
             try:
                 req = unserialize(msg)
                 assert isinstance(req['ID'], int)
             except:
                 logger.warning('Invalid compuation for deleting')
+                conn.close()
                 raise StopIteration
             cluster = self._clusters.get(req['ID'], None)
             if cluster is None:
                 # this cluster is closed
+                conn.close()
                 raise StopIteration
             cluster.zombie = True
             self.cleanup_computation(cluster)
         elif msg.startswith('FILEXFER:'):
             msg = msg[len('FILEXFER:'):]
-            yield _xfer_file_task(self, msg)
+            resp = yield _xfer_file_task(self, msg)
         elif msg.startswith('TERMINATE_JOB:'):
             msg = msg[len('TERMINATE_JOB:'):]
-            conn.close()
             try:
                 job = unserialize(msg)
             except:
                 logger.warning('Invalid job cancel message')
+                conn.close()
                 raise StopIteration
             cluster = self._clusters.get(job.compute_id, None)
             if not cluster:
                 logger.debug('Invalid job %s!', job.uid)
+                conn.close()
                 raise StopIteration
             compute = cluster._compute
             cluster.last_pulse = time.time()
@@ -867,7 +853,6 @@ class _Scheduler(object):
                                 cluster.pending_results -= 1
                         except:
                             logger.debug('Could not remove "%s"', result_file)
-                        raise StopIteration
                     else:
                         resp = serialize('Invalid job')
             except:
@@ -882,7 +867,7 @@ class _Scheduler(object):
             except:
                 logger.warning('Failed to send response to %s: %s',
                                str(addr), traceback.format_exc())
-            conn.close()
+        conn.close()
         # end of scheduler_task
 
     def timer_task(self, coro=None):
